@@ -21,7 +21,6 @@ from kazoo.protocol import paths as k_paths
 from kazoo.recipe import watchers
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
-import six
 import testtools
 from zake import fake_client
 from zake import utils as zake_utils
@@ -171,7 +170,7 @@ class ZakeJobboardTest(test.TestCase, ZookeeperBoardTestMixin):
             # Forcefully delete the owner from the backend storage to make
             # sure the job becomes unclaimed (this may happen if some admin
             # manually deletes the lock).
-            paths = list(six.iteritems(self.client.storage.paths))
+            paths = list(self.client.storage.paths.items())
             for (path, value) in paths:
                 if path in self.bad_paths:
                     continue
@@ -192,7 +191,7 @@ class ZakeJobboardTest(test.TestCase, ZookeeperBoardTestMixin):
             # Forcefully delete the lock from the backend storage to make
             # sure the job becomes unclaimed (this may happen if some admin
             # manually deletes the lock).
-            paths = list(six.iteritems(self.client.storage.paths))
+            paths = list(self.client.storage.paths.items())
             for (path, value) in paths:
                 if path in self.bad_paths:
                     continue
@@ -215,7 +214,7 @@ class ZakeJobboardTest(test.TestCase, ZookeeperBoardTestMixin):
 
             trashed = []
             jobs = []
-            paths = list(six.iteritems(self.client.storage.paths))
+            paths = list(self.client.storage.paths.items())
             for (path, value) in paths:
                 if path in self.bad_paths:
                     continue
@@ -244,14 +243,14 @@ class ZakeJobboardTest(test.TestCase, ZookeeperBoardTestMixin):
         # Remove paths that got created due to the running process that we are
         # not interested in...
         paths = {}
-        for (path, data) in six.iteritems(self.client.storage.paths):
+        for (path, data) in self.client.storage.paths.items():
             if path in self.bad_paths:
                 continue
             paths[path] = data
 
         # Check the actual data that was posted.
         self.assertEqual(1, len(paths))
-        path_key = list(six.iterkeys(paths))[0]
+        path_key = list(paths.keys())[0]
         self.assertTrue(len(paths[path_key]['data']) > 0)
         self.assertDictEqual({
             'uuid': posted_job.uuid,
@@ -294,3 +293,47 @@ class ZakeJobboardTest(test.TestCase, ZookeeperBoardTestMixin):
             self.assertRaises(excp.NotImplementedError,
                               self.board.register_entity,
                               entity_instance_2)
+
+    def test_connect_check_compatible(self):
+        # Valid version
+        client = fake_client.FakeClient()
+        board = impl_zookeeper.ZookeeperJobBoard(
+            'test-board', {'check_compatible': True},
+            client=client)
+        self.addCleanup(board.close)
+        self.addCleanup(self.close_client, client)
+
+        with base.connect_close(board):
+            pass
+
+        # Invalid version, no check
+        client = fake_client.FakeClient(server_version=(3, 2, 0))
+        board = impl_zookeeper.ZookeeperJobBoard(
+            'test-board', {'check_compatible': False},
+            client=client)
+        self.addCleanup(board.close)
+        self.addCleanup(self.close_client, client)
+
+        with base.connect_close(board):
+            pass
+
+        # Invalid version, check_compatible=True
+        client = fake_client.FakeClient(server_version=(3, 2, 0))
+        board = impl_zookeeper.ZookeeperJobBoard(
+            'test-board', {'check_compatible': True},
+            client=client)
+        self.addCleanup(board.close)
+        self.addCleanup(self.close_client, client)
+
+        self.assertRaises(excp.IncompatibleVersion, board.connect)
+
+        # Invalid version, check_compatible='False'
+        client = fake_client.FakeClient(server_version=(3, 2, 0))
+        board = impl_zookeeper.ZookeeperJobBoard(
+            'test-board', {'check_compatible': 'False'},
+            client=client)
+        self.addCleanup(board.close)
+        self.addCleanup(self.close_client, client)
+
+        with base.connect_close(board):
+            pass
